# coding: utf8
"""MMM-Facial-Recognition - MagicMirror Module
Face Recognition Testing Script
The MIT License (MIT)

Copyright (c) 2016 Paul-Vincent Roll (MIT License)
Based on work by Tony DiCola (Copyright 2013) (MIT License)

Run this script to test your training data.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import cv2  # OpenCV Library
import lib.face as face
import lib.config as config
import time
import os
import urllib
import RPi.GPIO as GPIO
import datetime


def Face_Detect(model, camera) :
    """ Детектируем пользователя Илона
        Если он на видео есть, то возвращаем True
	Иначе False
    """
    ret,frame=camera.read()
    #print ("Детектируем")
    image = cv2.cvtColor(frame, cv2.COLOR_RGB2GRAY)
    #cv2.imshow('Facial recognition', frame)
    faces = face.detect_face(image)
    if faces is not None:
        for i in range(0, len(faces)):
            x, y, w, h = faces[i]
            if config.RECOGNITION_ALGORITHM == 1:
                crop = face.crop(image, x, y, w, h)
            else:
                crop = face.resize(face.crop(image, x, y, w, h))

            label, confidence = model.predict(crop)
            if (label != -1 and label != 0):
                # If person is close to the camera use smaller POSITIVE_THRESHOLD
                if h > 190 and confidence < config.POSITIVE_THRESHOLD:
                    #print('User:' + config.users[label - 1])
                    user_name=config.users[label - 1]
                    if user_name=="Ilona":
                        return True
                # If person is further away from the camera but POSITIVE_THRESHOLD is still under 40 assume it is the person
                elif h <= 190 and confidence < config.POSITIVE_THRESHOLD:
                    #print('User:' + config.users[label - 1])
                    user_name=config.users[label - 1]
                    if user_name=="Ilona":
                        return True
    return False


# Use BCM GPIO references instead of physical pin numbers
GPIO.setmode(GPIO.BOARD)
# Define GPIO to use on Pi
GPIO_PIR = 11

# Set pin as input
GPIO.setup(GPIO_PIR,GPIO.IN) # Echo
Current_State = 0
Previous_State = 0

# Load training data into model
print('Loading training data...')

if config.RECOGNITION_ALGORITHM == 1:
    print ("ALGORITHM: LBPH")
    model = cv2.createLBPHFaceRecognizer(threshold=config.POSITIVE_THRESHOLD)
elif config.RECOGNITION_ALGORITHM == 2:
    print ("ALGORITHM: Fisher")
    model = cv2.createFisherFaceRecognizer(threshold=config.POSITIVE_THRESHOLD)
else:
    print ("ALGORITHM: Eigen")
    model = cv2.createEigenFaceRecognizer(threshold=config.POSITIVE_THRESHOLD)

model.load("training.xml")
print('Training data loaded!')

#camera = config.get_camera()
camera=cv2.VideoCapture(0)
#time.sleep(1)   # give the camera a second to warm up

check_time=0

try:
    # Loop until PIR output is 0
    while GPIO.input(GPIO_PIR)==1:
        Current_State = 0
    # Loop until users quits with CTRL-C
    while True :
        # Read PIR state
        Current_State = GPIO.input(GPIO_PIR)
        if Current_State==1 and Previous_State==0:
            # PIR is triggered
            print( datetime.datetime.now()," Motion detected!")
            link="http://127.0.0.1/objects/?object=HallMotion&op=m&m=motionDetected&"
            f=urllib.urlopen(link)
            # Record previous state
            Previous_State=1
            # Face recognize started
            rec_start=time.time() 
            ilona_rec=0
            while True:
                if check_time+3600>time.time() :
                    break
                if Face_Detect(model, camera) :
                    ilona_rec += 1 
                if ilona_rec>2 :
                    link="http://127.0.0.1/objects/?script=Ilona_Greeting"
                    f=urllib.urlopen(link)
                    check_time=time.time()
                    break
                cur_time=time.time()
                if cur_time>rec_start+60 :
                    break 
	  
        elif Current_State==0 and Previous_State==1:
            Previous_State=0
        time.sleep(10)
except KeyboardInterrupt:
    print " Quit"
    # Reset GPIO settings
    GPIO.cleanup()



# Release camera and close windows
camera.stop()
cv2.destroyAllWindows()
